==> typespeed-0.5.1.tar.gz: typespeed-0.5.1/testsuite/testloadwords.c <==
/*
 *  typespeed - measures your typing speed
 *  Copyright (C) 1999-2003   Jani Ollikainen  <bestis@iki.fi>
 *                          & Jaakko Manelius  <jman@iki.fi>
 *  Copyright (C) 2006   Tobias Stoeckmann  <tobias@bugol.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * return values
 *
 * 0:	success
 * 1:	memory failure
 * 101:	usage
 * 102:	loadwords returns 1
 * 103:	null pointer in word (< wordcount)
 * 104:	pointer in word (> wordcount)
 *
 * -w option:
 * wordcount
 */
#include <sys/param.h>
#include <sys/types.h>

#include <ctype.h>
#include <curses.h>
#include <dirent.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

size_t	WORD_MAX;
char	**word;
int	wordcount;

/*
 * COPY, DON'T CHANGE ANYTHING HERE!
 */

/* wrapper for malloc */
void *
xmalloc(size_t size)
{
	void *pointer;
	if ((pointer = malloc(size)) == NULL) {
		fputs("Cannot allocate memory!\n", stderr);
		exit(1);
	}
	return pointer;
}

void
freewords()
{
	int i;
	for (i = 0; i < wordcount; i++) {
		free(word[i]);
		word[i] = NULL;
	}
	wordcount = 0;
}

/*
 * Reads in max WORD_MAX words out of file "filename" into word. If
 * less than 22 words are parsed, all words are freed and 1 is returned.
 * Words cannot be longer than 20 chars. No word duplicates allowed.
 *
 * First line of file "filename" will be ignored (description).
 *
 * Returns 0 on success or 1 on error.
 */
int
loadwords(char *filename)
{
	FILE *wordlist;
	char *myword[WORD_MAX];
	char *p;
	char newword[20 + 3];
	int i, ignore, j;
	size_t mycount;

	ignore = 1;

	if ((wordlist = fopen(filename, "r")) == NULL)
		return 1;

	memset(myword, 0, sizeof(myword));
	mycount = 0;
	while (fgets(newword, sizeof(newword), wordlist) != NULL) {
		/*
		 * we won't freak out about words which are against
		 * the rule (e.g. too long). Instead we will silently
		 * ignore them.
		 */
		if ((p = strchr(newword, '\n')) == NULL) {
			ignore = 1;
			continue;
		}
		if (ignore) {
			ignore = 0;
			continue;
		}

		*p = '\0';
		if ((p = strchr(newword, '\r')) != NULL)
			*p = '\0';

		if (strlen(newword) > 20 || !strlen(newword))
			continue;

		for (ignore = 0, p = newword; *p != '\0'; p++)
			if (!isprint(*p) || isspace(*p)) {
				ignore = 1;
				break;
			}
		if (ignore) {
			ignore = 0;
			continue;
		}

		if (mycount >= WORD_MAX)
			break;

		myword[mycount] = (char *)xmalloc(strlen(newword) + 1);
		strcpy(myword[mycount++], newword);
	}
	fclose(wordlist);

	for (i = 0; i < mycount; i++) {
		for (j = 0; j < WORD_MAX; j++) {
			if (word[j] == NULL) {
				if (wordcount < WORD_MAX) {
					wordcount++;
					word[j] = myword[i];
				}
				break;
			}
			if (!strcmp(myword[i], word[j])) {
				free(myword[i]);
				break;
			}
		}
		if (wordcount == WORD_MAX)
			break;
	}

	if (wordcount < 22) {
		/* not enough words */
		freewords();
		return 1;
	}
	return 0;
}

/*
 * TEST FUNCTIONS
 */

void
usage(void)
{
	fputs("usage: testloadwords [-cvw] [-l loops] [-s seconds] [-W WORD_MAX] wordlist1 ...\n", stderr);
	exit(101);
}

int
main(int argc, char **argv)
{
	int ch, clear, i, loop, retwordcount, seconds, verbose;

	setlocale(LC_ALL, "");

	clear = retwordcount = loop = seconds = verbose = 0;
	WORD_MAX = 10000;
        while ((ch = getopt(argc, argv, "cl:s:vW:w")) != -1)
                switch (ch) {
		case 'c':
			clear = 1;
			break;
		case 'l':
			loop = atoi(optarg);
		case 's':
			seconds = atoi(optarg);
		case 'v':
			verbose = 1;
			break;
		case 'w':
			retwordcount = 1;
			break;
		case 'W':
			WORD_MAX = atoi(optarg);
			break;
		case '?':
		default:
			usage();
			break;
		}
	argc -= optind;
        argv += optind;

	if (!argc)
		usage();

	if ((word = calloc(WORD_MAX, sizeof(char *))) == NULL) {
		fputs("Cannot allocate memory!", stderr);
		exit(1);
	}
	wordcount = 0;

	do {
		if (seconds)
			sleep(seconds);
		while (*argv != NULL) {
			if (verbose)
				puts(*argv);
			if (loadwords(*argv)) {
				if (verbose)
					fputs("Failure in loadwords (1)\n", stderr);
				exit(102);
			}
			if (verbose)
				printf("wordcount is %d\n", wordcount);
			if (clear)
				freewords();

			for (i = 0; i < wordcount; i++)
				if (word[i] == NULL) {
					if (verbose)
						fputs("NULL pointer (< wordcount)!", stderr);
					exit(103);
				}
			for (i = wordcount; i < WORD_MAX; i++)
				if (word[i] != NULL) {
					if (verbose)
						fputs("pointer (> wordcount)!", stderr);
					exit(104);
				}
			argv++;
		}
	} while (loop--);

	if (retwordcount)
		exit(wordcount);
	exit(0);
}

==> typespeed-0.5.2.tar.gz: typespeed-0.5.2/testsuite/testloadwords.c <==
/*
 *  typespeed - measures your typing speed
 *  Copyright (C) 1999-2003   Jani Ollikainen  <bestis@iki.fi>
 *                          & Jaakko Manelius  <jman@iki.fi>
 *  Copyright (C) 2006   Tobias Stoeckmann  <tobias@bugol.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * return values
 *
 * 0:	success
 * 1:	memory failure
 * 101:	usage
 * 102:	loadwords returns 1
 * 103:	null pointer in word (< wordcount)
 * 104:	pointer in word (> wordcount)
 *
 * -w option:
 * wordcount
 */
#include <sys/param.h>
#include <sys/types.h>

#include <ctype.h>
#include <curses.h>
#include <dirent.h>
#include <err.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

size_t	WORD_MAX;
char	**word;
int	wordcount;

char	worddir[MAXPATHLEN] = ".";

void
endcursestuff(void)
{
	return;
}

/*
 * COPY, DON'T CHANGE ANYTHING HERE!
 */

/* wrapper for malloc */
void *
xmalloc(size_t size)
{
	void *pointer;
	if ((pointer = malloc(size)) == NULL)
		err(1, "malloc()");
	return pointer;
}

/* Frees all allocated memory in word. */
void
freewords(void)
{
	while (wordcount) {
		wordcount--;
		free(word[wordcount]);
		word[wordcount] = NULL;
	}
}

/*
 * Reads in max WORD_MAX words out of file "filename" into word. If
 * less than 22 words are parsed, all words are freed and 1 is returned.
 * Words cannot be longer than 19 chars. No word duplicates allowed.
 *
 * First line of file "filename" will be ignored (description).
 *
 * XXX: loadwords is able to add words to an already populated
 * word array, I think this is not needed anywhere, so this
 * functionality will be removed.
 *
 * Returns
 * 0 on success
 * 1 on snprintf failure (file path too long)
 * 2 cannot open file (errno set by fopen)
 * 3 not enough words in wordfile
 */
int
loadwords(char *filename)
{
	FILE *wordlist;
	char *myword[WORD_MAX], *p;
	char newword[19 + 3], wordpath[MAXPATHLEN];
	int ignore, j, n;
	size_t i, mycount;

	n = snprintf(wordpath, sizeof(wordpath), "%s/%s", worddir, filename);
	if (n < 1 || (size_t)n > sizeof(wordpath) - 1)
		return 1;

	if ((wordlist = fopen(wordpath, "r")) == NULL)
		return 2;

	memset(myword, 0, sizeof(myword));
	mycount = 0;
	ignore = 1;
	while (fgets(newword, sizeof(newword), wordlist) != NULL) {
		/*
		 * We won't freak out about words which are against
		 * the rule (e.g. too long). Instead we will silently
		 * ignore them.
		 */
		if ((p = strchr(newword, '\n')) == NULL) {
			ignore = 1;
			continue;
		}
		if (ignore) {
			ignore = 0;
			continue;
		}

		*p = '\0';
		if ((p = strchr(newword, '\r')) != NULL)
			*p = '\0';

		if (strlen(newword) > 19 || !strlen(newword))
			continue;

		for (ignore = 0, p = newword; *p != '\0'; p++)
			if (!isprint(*p) || isspace(*p)) {
				ignore = 1;
				break;
			}
		if (ignore) {
			ignore = 0;
			continue;
		}

		if (mycount >= WORD_MAX)
			break;

		if ((myword[mycount++] = strdup(newword)) == NULL)
			err(1, "strdup()");
	}
	if (fclose(wordlist) == EOF) {
		endcursestuff();
		err(1, "fclose(%s)", wordpath);
	}

	for (i = 0; i < mycount; i++) {
		for (j = 0; j < WORD_MAX; j++) {
			if (word[j] == NULL) {
				if (wordcount < WORD_MAX) {
					wordcount++;
					word[j] = myword[i];
				}
				break;
			}
			if (!strcmp(myword[i], word[j])) {
				free(myword[i]);
				break;
			}
		}
		if (wordcount == WORD_MAX)
			break;
	}

	if (wordcount < 22) {
		/* not enough words */
		freewords();
		return 3;
	}
	return 0;
}

/*
 * TEST FUNCTIONS
 */

void
usage(void)
{
	fputs("usage: testloadwords [-cvw] [-l loops] [-s seconds] [-W WORD_MAX] wordlist1 ...\n", stderr);
	exit(101);
}

int
main(int argc, char **argv)
{
	int ch, clear, i, loop, retwordcount, seconds, verbose;

	setlocale(LC_ALL, "");

	clear = retwordcount = loop = seconds = verbose = 0;
	WORD_MAX = 10000;
        while ((ch = getopt(argc, argv, "cl:s:vW:w")) != -1)
                switch (ch) {
		case 'c':
			clear = 1;
			break;
		case 'l':
			loop = atoi(optarg);
		case 's':
			seconds = atoi(optarg);
		case 'v':
			verbose = 1;
			break;
		case 'w':
			retwordcount = 1;
			break;
		case 'W':
			WORD_MAX = atoi(optarg);
			break;
		case '?':
		default:
			usage();
			break;
		}
	argc -= optind;
        argv += optind;

	if (!argc)
		usage();

	if ((word = calloc(WORD_MAX, sizeof(char *))) == NULL) {
		fputs("Cannot allocate memory!", stderr);
		exit(1);
	}
	wordcount = 0;

	do {
		if (seconds)
			sleep(seconds);
		while (*argv != NULL) {
			if (verbose)
				puts(*argv);
			if (loadwords(*argv)) {
				if (verbose)
					fputs("Failure in loadwords (1)\n", stderr);
				exit(102);
			}
			if (verbose)
				printf("wordcount is %d\n", wordcount);
			if (clear)
				freewords();

			for (i = 0; i < wordcount; i++)
				if (word[i] == NULL) {
					if (verbose)
						fputs("NULL pointer (< wordcount)!", stderr);
					exit(103);
				}
			for (i = wordcount; i < WORD_MAX; i++)
				if (word[i] != NULL) {
					if (verbose)
						fputs("pointer (> wordcount)!", stderr);
					exit(104);
				}
			argv++;
		}
	} while (loop--);

	if (retwordcount)
		exit(wordcount);
	exit(0);
}
